/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date;

import org.cobogw.gwt.user.client.ui.RoundedPanel;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;

import eu.future.earth.gwt.client.FtrGwtDateCss;
import eu.future.earth.gwt.client.date.DateEvent.DateEventActions;

public abstract class AbstractWholeDayField<T> extends EventPanel<T> implements ClickHandler, HasDateEventHandlers<T> {

	private HorizontalPanel whole = new HorizontalPanel();

	private SimplePanel base;

	protected Label title = new Label();

	private HTML drag = new HTML();

	private TimeEventRendererVertical<T> dateRenderer = null;

	public AbstractWholeDayField(TimeEventRendererVertical<T> newRenderer, T data) {
		super(newRenderer, data);
		dateRenderer = newRenderer;
		if (dateRenderer.getEventCornerSize() > 0) {
			base = new RoundedPanel(whole, RoundedPanel.ALL, dateRenderer.getEventCornerSize());
		} else {
			base = new SimplePanel();
			base.setWidget(whole);
		}
		drag.setHTML("&nbsp;&nbsp;&nbsp;");
		drag.setWidth("100%");
		whole.add(title);
		whole.setSpacing(0);
		whole.add(drag);
		whole.setWidth("100%");
		whole.setCellWidth(title, "10%");
		whole.setCellWidth(drag, "90%");
		title.addClickHandler(this);
		super.setWidget(base);
		this.setWidth("100%");
		title.setWordWrap(false);
		setEventStyleName(FtrGwtDateCss.EVENT_WHOLEDAY_PANEL);
	}

	public int getContentHeight() {
		return contentHeight;
	}

	private int contentHeight = 15;

	public void setContentHeight(int contentHeight) {
		if (contentHeight > 0) {
			this.contentHeight = contentHeight;
		}
	}

	public void setWidth(int width) {
		if (width > 20) {
			title.setWidth(width - 20 + "px");
			super.setWidth(width + "px");
		}
	}

	public void setEventStyleName(String newStyleName) {
		if (base instanceof RoundedPanel) {
			((RoundedPanel) base).setCornerStyleName(newStyleName);
		} else {
			base.setStyleName(newStyleName);
		}
		whole.setStyleName(newStyleName);
		title.setStyleName(newStyleName);
		drag.setStyleName(newStyleName);
	}

	public Widget getDraggableItem() {
		return drag;
	}

	public Widget getPanel() {
		return whole;
	}

	public void setDraggable(boolean newState) {

	}

	public void onClick(ClickEvent widget) {
		if (widget.getSource() == title) {
			DateEvent.fire(this, DateEventActions.EDIT, getValue());
		}
	}

	public HandlerRegistration addDateEventHandler(DateEventListener<? extends T> handler) {
		return addHandler(handler, DateEvent.getType());
	}

	public void setTitle(String newTitle) {
		title.setText(newTitle);
	}

}
