/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date;

import java.util.Date;

import com.google.gwt.event.shared.GwtEvent;

public class DateEvent<T> extends GwtEvent<DateEventListener<T>> {

	private static final long serialVersionUID = -3798647759325157673L;

	public enum DateEventActions {

		ADD,

		ADD_FAILED,

		ADD_DONE,

		UPDATE,

		UPDATE_FAILED,

		UPDATE_DONE,

		REMOVE,

		REMOVE_FAILED,

		REMOVE_DONE,

		DRAG_DROP,

		EDIT,

		SELECT_DAY,

		SELECT_MONTH,

		RELOAD,

		GO_TO_DAY_VIEW,

		DAY_SELECTED

	}

	private T data = null;

	private Date date = null;

	private DateEventActions command = DateEventActions.ADD;

	public DateEvent() {
		super();
	}

	/**
	 * Handler type.
	 */
	private static Type<DateEventListener<?>> TYPE;

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, Date navigator, T token) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateEvent<T> event = new DateEvent<T>(navigator, token);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, Date navigator) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateEvent<T> event = new DateEvent<T>(navigator);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, DateEvent<T> navigator) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			source.fireEvent(navigator);
			return navigator;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, Date navigator, DateEventActions action) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateEvent<T> event = new DateEvent<T>(navigator, action);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, DateEventActions action) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateEvent<T> event = new DateEvent<T>(action);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, Date navigator, DateEventActions action, T newData) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateEvent<T> event = new DateEvent<T>(navigator, action, newData);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static <T> DateEvent<T> fire(HasDateEventHandlers<T> source, DateEventActions action, T newData) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateEvent<T> event = new DateEvent<T>(action, newData);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	public T getData() {
		return data;
	}

	public Date getDate() {
		return date;
	}

	public DateEventActions getCommand() {
		return command;
	}

	/**
	 * Gets the type associated with this event.
	 * 
	 * @return returns the handler type
	 */
	public static <T> Type<DateEventListener<?>> getType() {
		if (TYPE == null) {
			TYPE = new Type<DateEventListener<?>>();
		}
		return TYPE;
	}

	protected DateEvent(Date newDate, T token2) {
		super();
		date = newDate;
		data = token2;
	}

	protected DateEvent(Date newDate, DateEventActions newAction) {
		super();
		date = newDate;
		command = newAction;
	}

	protected DateEvent(DateEventActions newAction) {
		super();
		command = newAction;
	}

	protected DateEvent(DateEventActions newAction, T newData) {
		super();
		command = newAction;
		data = newData;
	}

	protected DateEvent(Date newDate, DateEventActions newAction, T newData) {
		super();
		date = newDate;
		command = newAction;
		data = newData;
	}

	protected DateEvent(Date newDate) {
		super();
		date = newDate;
	}

	@SuppressWarnings({
			"unchecked", "rawtypes"
	})
	@Override
	public final Type<DateEventListener<T>> getAssociatedType() {
		return (Type) TYPE;
	}

	protected void dispatch(DateEventListener<T> handler) {
		handler.handleDateEvent(this);
	}

}
