/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.horizontal;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.Widget;

import eu.future.earth.gwt.client.FtrGwtDateCss;

public abstract class BaseHorizontalRowPanel<T, M> extends FocusPanel {

	protected HorizontalDateRenderer<T, M> renderer;

	protected int startHour = 0;

	protected int endHour = 24;

	private ExtendedHorizontalAbsolutePanel<T, M> body = null;

	public BaseHorizontalRowPanel(HorizontalDateRenderer<T, M> newRenderer) {
		super();
		this.renderer = newRenderer;
		body = new ExtendedHorizontalAbsolutePanel<T, M>(this);
		this.setWidget(body);
		body.setStyleName(FtrGwtDateCss.DATE_DAY_FULL);
		startHour = renderer.getStartHour();
		endHour = renderer.getEndHour();

	}

	public HorizontalDateRenderer<T, M> getRenderer() {
		return renderer;
	}

	protected int getDailyIntervals() {
		return (endHour - startHour) * renderer.getIntervalsPerHour();
	}

	public boolean removeFromBody(Widget w) {
		return body.removeNormal(w);
	}

	public void add(Widget w, int left, int top) {
		body.addNormal(w, left, top);
	}

	public AbsolutePanel getBody() {
		return body;
	}

}
