/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.horizontal;

import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.VerticalPanel;

import eu.future.earth.gwt.client.FtrGwtDateCss;

public class HorizontalHourPanel<T, M> extends BaseHorizontalRowPanel<T, M> {

	public HorizontalHourPanel(HorizontalDateRenderer<T, M> renderer, boolean useCss) {
		super(renderer);
		if (useCss) {
			setStyleName(FtrGwtDateCss.HOUR_PANEL);
		} else {
			setStyleName(FtrGwtDateCss.HOUR_PANEL_NOBACKGROUND);
		}
		int hourHeight = renderer.getIntervalsPerHour() * renderer.getIntervalWidth();
		int intervalMinutes = (60 / renderer.getIntervalsPerHour());
		int x = 0;

		for (int hour = startHour; hour < endHour; hour++) {
			VerticalPanel drow = new VerticalPanel();
			drow.setStyleName(FtrGwtDateCss.HOUR_NO_BORDER);
			drow.setSize("30px", renderer.getRowHeight() + "px");

			int displayHour = hour;
			String displaySuffix = "";
			if (!renderer.show24HourClock()) {
				if (hour < 12) {
					displaySuffix = " AM";
				} else {
					displaySuffix = " PM";
				}

				if (hour == 0) {
					displayHour = 12;
				} else {
					if (hour > 12) {
						displayHour = hour - 12;
					}
				}
			}

			for (int interval = 0; interval < (renderer.showIntervalTimes() ? renderer.getIntervalsPerHour() : 1); interval++) {
				int minute = (interval * intervalMinutes);
				String displayMinute = (minute < 10) ? "0" + minute : "" + minute;
				Label label = new Label(displayHour + ":" + displayMinute + displaySuffix);
				label.setStyleName(FtrGwtDateCss.HOUR_LABEL);
				drow.add(label);
			}

			super.add(drow, x, 0);
			x = x + hourHeight;
		}

	}

	public void repaint() {
	}
}
