/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.horizontal;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;

import com.allen_sauer.gwt.dnd.client.AbstractDragController;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Widget;

import eu.future.earth.gwt.client.date.DateEvent.DateEventActions;
import eu.future.earth.gwt.client.date.horizontal.HorizontalDayField.DirectionConstant;

final public class HorizontalItemResizeDragController<T, M> extends AbstractDragController {

	public HorizontalItemResizeDragController(HorizontalItemRow<T, M> boundaryPanel) {
		super(boundaryPanel.getBody());
		super.setBehaviorDragStartSensitivity(1);
		parent = boundaryPanel;
	}

	private HorizontalDayField<T, M> resizePanel = null;

	private HorizontalItemRow<T, M> parent = null;

	private HashMap<Widget, DirectionConstant> directionMap = new HashMap<Widget, DirectionConstant>();

	public void dragStart() {
		super.dragStart();
		resizePanel = getDayField(context.draggable);
		parent = getDayPanel(resizePanel);
	}

	@SuppressWarnings("unchecked")
	private HorizontalDayField<T, M> getDayField(Widget current) {
		Widget parent = current.getParent();
		if (parent instanceof HorizontalDayField<?, ?>) {
			return (HorizontalDayField<T, M>) parent;
		} else {
			if (parent != null) {
				return getDayField(parent);
			} else {
				return null;
			}
		}
	}

	@SuppressWarnings("unchecked")
	private HorizontalItemRow<T, M> getDayPanel(Widget current) {
		Widget parent = current.getParent();
		if (parent instanceof HorizontalItemRow<?, ?>) {
			return (HorizontalItemRow<T, M>) parent;
		} else {
			if (parent != null) {
				return getDayPanel(parent);
			} else {
				return null;
			}
		}
	}

	private Calendar helper = new GregorianCalendar(); // NOPMD;

	protected DirectionConstant getDirection(Widget draggable) {
		return (DirectionConstant) directionMap.get(draggable);
	}

	public void makeDraggable(Widget widget, HorizontalDayField.DirectionConstant direction) {
		super.makeDraggable(widget);
		directionMap.put(widget, direction);
	}

	@SuppressWarnings("unchecked")
	public void dragMove() {
		int direction = ((HorizontalItemResizeDragController<T, M>) context.dragController).getDirection(context.draggable).directionBits;
		final int he = resizePanel.getWidth();
		if ((direction & HorizontalDayField.DIRECTION_EAST) != 0) {
			int delta = context.desiredDraggableX - context.draggable.getAbsoluteLeft();
			if (delta != 0) {
				final int newHeight = he + delta;
				helper.setTime(resizePanel.getStart());
				// make sure we stay on an perfect minute
				helper.set(Calendar.SECOND, 0);
				helper.set(Calendar.MILLISECOND, 0);
				// add however many minutes the current interval snap represents to the start time
				helper.add(Calendar.MINUTE, parent.calculateIntervalSnapForX(newHeight) * (60 / parent.getRenderer().getIntervalsPerHour()));

				resizePanel.setEndTime(helper.getTime());
				resizePanel.repaintTime();
				resizePanel.setWidth(newHeight);
			}
		}
	}

	public void dragEnd() {
		super.dragEnd();
		if (parent == null) {
			Window.alert("Parent is null");
		}
		parent.notifyParentOfUpdate(DateEventActions.UPDATE, resizePanel.getStart(), resizePanel.getValue());
	}

}
