package eu.future.earth.gwt.client.date.horizontal;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.MouseDownEvent;
import com.google.gwt.event.dom.client.MouseDownHandler;
import com.google.gwt.event.dom.client.MouseMoveEvent;
import com.google.gwt.event.dom.client.MouseMoveHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseUpEvent;
import com.google.gwt.event.dom.client.MouseUpHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.Widget;

import eu.future.earth.gwt.client.FtrGwtDateCss;
import eu.future.earth.gwt.client.date.DateEvent;
import eu.future.earth.gwt.client.date.DateEventListener;
import eu.future.earth.gwt.client.date.DateUtils;
import eu.future.earth.gwt.client.date.EventPanel;
import eu.future.earth.gwt.client.date.HasDateEventHandlers;
import eu.future.earth.gwt.client.date.DateEvent.DateEventActions;

public class HorizontalItemRow<T, M> extends BaseHorizontalRowPanel<T, M> implements MouseDownHandler, MouseMoveHandler, MouseUpHandler, MouseOutHandler, HasDateEventHandlers<T> {

	private DateTimeFormat formatterTime = null;

	private ArrayList<FocusPanel> hh = new ArrayList<FocusPanel>();

	private String startTimeString = null;

	private Calendar helper = new GregorianCalendar();

	private Calendar theDay = new GregorianCalendar();

	private HorizontalViewPanelNoDays<T, M> parent = null;

	private ArrayList<EventPanel<T>> events = new ArrayList<EventPanel<T>>();

	private HorizontalItemResizeDragController<T, M> resizeDragController = null;

	private M master = null;

	private HorizontalTimeCalculator calculator = null;

	public HorizontalItemRow(HorizontalDateRenderer<T, M> newRenderer, HorizontalViewPanelNoDays<T, M> newParent, M newMaster) {
		super(newRenderer);
		parent = newParent;
		master = newMaster;
		calculator = new HorizontalTimeCalculator(newRenderer);
		super.setStyleName(FtrGwtDateCss.HOR_ROW);
		formatterTime = DateTimeFormat.getFormat(renderer.show24HourClock() ? "HH:mm" : "hh:mm a");
		if (renderer.enableDragAndDrop()) {
			GWT.log("Resize", null);
			resizeDragController = new HorizontalItemResizeDragController<T, M>(this);
		}
		int intervalMinutes = (60 / renderer.getIntervalsPerHour());
		int y = 0;
		for (int hour = startHour; hour < endHour; hour++) {

			int displayHour = hour;
			String displaySuffix = "";
			if (!renderer.show24HourClock()) {
				if (hour < 12) {
					displaySuffix = " AM";
				} else {
					displaySuffix = " PM";
				}
				if (hour == 0) {
					displayHour = 12;
				} else {
					if (hour > 12) {
						displayHour = hour - 12;
					}
				}
			}

			for (int interval = 0; interval < renderer.getIntervalsPerHour(); interval++) {
				final FocusPanel drow = new FocusPanel();
				int minute = (interval * intervalMinutes);
				String displayMinute = (minute < 10) ? "0" + minute : "" + minute;
				drow.setTitle(displayHour + ":" + displayMinute + displaySuffix);
				if (interval == 0) {
					drow.setStyleName(FtrGwtDateCss.HOR_HOUR_INTERVAL_START);
				} else {
					drow.setStyleName(FtrGwtDateCss.HOR_HOUR_INTERVAL_BETWEEN);
				}
				drow.setSize(renderer.getIntervalWidth() + "px", renderer.getRowHeight() + "px");
				drow.addMouseDownHandler(this);
				drow.addMouseMoveHandler(this);
				drow.addMouseUpHandler(this);
				hh.add(drow);
				super.add(drow, y, 0);
				y = y + renderer.getIntervalWidth();
			}
		}
		this.addMouseOutHandler(this);

	}



	public List<T> getData() {
		List<T> result = new ArrayList<T>();
		for (int i = 0; i < events.size(); i++) {
			result.add(events.get(i).getValue());
		}
		return result;
	}

	public int numberOfEvents() {
		return events.size();
	}

	public void destroy() {
		if (getParent() != null) {
			removeFromParent();
		}
	}

	public void clearEvents() {
		for (int i = 0; i < events.size(); i++) {
			final EventPanel<T> ev = events.get(i);
			super.removeFromBody(ev);
		}
		events.clear();
	}

	public void setDay(Date newDay) {
		theDay.setTime(newDay);
	}

	public void addEventByDrop(HorizontalDayField<T, M> newEvent) {
		removeEvent(newEvent.getValue()); // Be safe
		newEvent.addStyleName(FtrGwtDateCss.HOR_DND_GETTING_STARTED_LABEL);
		helper.setTime(newEvent.getStart());
		helper.set(Calendar.YEAR, theDay.get(Calendar.YEAR));
		helper.set(Calendar.MONTH, theDay.get(Calendar.MONTH));
		helper.set(Calendar.DAY_OF_MONTH, theDay.get(Calendar.DAY_OF_MONTH));
		newEvent.setStart(helper.getTime());

		if (newEvent.getEnd() != null) {
			helper.setTime(newEvent.getEnd());
			helper.set(Calendar.YEAR, theDay.get(Calendar.YEAR));
			helper.set(Calendar.MONTH, theDay.get(Calendar.MONTH));
			helper.set(Calendar.DAY_OF_MONTH, theDay.get(Calendar.DAY_OF_MONTH));
			newEvent.setEndTime(helper.getTime());
		}

		paintEvent(newEvent);
		DateEvent.fire(this, DateEventActions.DRAG_DROP, newEvent.getValue());
		events.add(newEvent);
		newEvent.setResizeDragController(resizeDragController);
		// resizeDragController.makeDraggable(newEvent);

	}

	public HandlerRegistration addDateEventHandler(DateEventListener<? extends T> handler) {
		return addHandler(handler, DateEvent.getType());
	}

	public void notifyParentOfUpdate(DateEventActions command, Date theDate, T data) {
		DateEvent.fire(this, theDate, command, data);
	}

	@SuppressWarnings("unchecked")
	public void addEvent(EventPanel<T> newEvent) {
		removeEvent(newEvent.getValue()); // Be safe
		if (renderer.enableDragAndDrop()) {
			newEvent.addStyleName(FtrGwtDateCss.HOR_DND_GETTING_STARTED_LABEL);
			if (newEvent instanceof HorizontalDayField<?, ?>) {
				HorizontalDayField<T, M> real = (HorizontalDayField<T, M>) newEvent;
				GWT.log("Resize for event", null);
				real.setResizeDragController(resizeDragController);
			}
		}
		events.add(newEvent);
		paintEvent(newEvent);
	}

	public boolean isMaster(T newEvent) {
		if (master == null) {
			return false;
		}
		return renderer.isMaster(master, newEvent);
	}

	public boolean isRowForMaster(M newEvent) {
		if (master == null) {
			return false;
		}
		return renderer.isSameMaster(master, newEvent);
	}

	public EventPanel<T> removeEvent(T event) {
		for (int i = 0; i < events.size(); i++) {
			final EventPanel<T> ev = events.get(i);
			if (renderer.equal(ev.getValue(), event)) {
				super.removeFromBody(ev);
				events.remove(i);
				repaint();
				return ev;
			}
		}
		return null;
	}

	public void repaint() {
		for (int i = 0; i < events.size(); i++) {
			final EventPanel<T> ev = events.get(i);
			super.removeFromBody(ev);
			paintEvent(ev);
		}
	}

	@SuppressWarnings("unchecked")
	public void updateEvent(EventPanel<T> newEvent) {
		EventPanel<T> found = null;
		for (int i = 0; i < events.size(); i++) {
			final EventPanel<T> test = events.get(i);
			if (renderer.equal(test.getValue(), newEvent.getValue())) {
				found = test;
			}
		}
		if (found == null) {
			if (renderer.enableDragAndDrop()) {
				newEvent.addStyleName(FtrGwtDateCss.HOR_DND_GETTING_STARTED_LABEL);
				if (newEvent instanceof HorizontalDayField<?, ?>) {
					HorizontalDayField<T, M> real = (HorizontalDayField<T, M>) newEvent;
					real.setResizeDragController(resizeDragController);
				}
			}
			events.add(newEvent);

		} else {
			// Repaint
		}
		repaint();
	}

	public void onMouseDown(MouseDownEvent event) {
		if (event.getSource() instanceof Widget) {
			Widget widget = (Widget) event.getSource();
			if (widget != this) {
				startTimeString = widget.getTitle();
			}
		}
	}

	public void onMouseOut(MouseOutEvent event) {
		if (event.getSource() instanceof Widget) {
			Widget widget = (Widget) event.getSource();
			if (widget == this) {
				startTimeString = null;
				clearSelection();
			}
		}
	}

	public void onMouseMove(MouseMoveEvent event) {
		if (event.getSource() instanceof Widget) {
			Widget widget = (Widget) event.getSource();
			if (startTimeString != null && widget != this) {
				widget.addStyleName(FtrGwtDateCss.HOR_HOURS_SELECTED);
			}
		}
	}

	private void paintEvent(EventPanel<T> newEvent) {
		int x = calculator.getTimePos(newEvent.getStart());
		if (!DateUtils.isSameDay(theDay.getTime(), newEvent.getStart())) {
			x = 0;
		}
		int x2 = x + (renderer.getIntervalWidth() * renderer.getIntervalsPerHour());
		if (newEvent.getEnd() != null) {
			if (DateUtils.isSameDay(theDay.getTime(), newEvent.getEnd())) {
				x2 = calculator.getTimePos(newEvent.getEnd());
			} else {
				x2 = calculator.calculateXForHoursMinutes(renderer.getEndHour(), 0);
			}
		}
		newEvent.setPosY(0);
		newEvent.repaintTime();
		newEvent.setContentHeight(renderer.getRowHeight());
		newEvent.setWidth(x2 - x);
		super.add(newEvent, x, 0);
	}

	public void onMouseUp(MouseUpEvent event) {
		if (event.getSource() instanceof Widget) {
			Widget widget = (Widget) event.getSource();
			// startTime
			if (startTimeString != null && widget != this) {
				// if (startTime.indexOf("AM") > -1 || startTime.indexOf("PM") > -1) {
				// startTime = startTime.substring(0, startTime.length() - 3);
				// }
				helper.setTime(theDay.getTime());
				final Calendar helperTime = new GregorianCalendar();

				final Date newDate = formatterTime.parse(startTimeString);
				helperTime.setTime(newDate);
				// Transplant the hour and minutes
				helper.set(Calendar.HOUR_OF_DAY, helperTime.get(Calendar.HOUR_OF_DAY));
				helper.set(Calendar.MINUTE, helperTime.get(Calendar.MINUTE));
				helper.set(Calendar.SECOND, 0);
				helper.set(Calendar.MILLISECOND, 0);
				final Date start = helper.getTime();
				final Date end;
				// add one interval's worth of time to get to the END of the selected interval

				String endTimeString = widget.getTitle();
				if (!startTimeString.equalsIgnoreCase(endTimeString)) {
					Date endTime = formatterTime.parse(endTimeString);
					// If drag is backwards, hack to reverse times shown in display.
					helperTime.setTime(endTime);
					helperTime.add(Calendar.MINUTE, 60 / renderer.getIntervalsPerHour());
					// Transplant the hour and minutes
					helper.set(Calendar.HOUR_OF_DAY, helperTime.get(Calendar.HOUR_OF_DAY));
					helper.set(Calendar.MINUTE, helperTime.get(Calendar.MINUTE));
					helper.set(Calendar.SECOND, 0);
					helper.set(Calendar.MILLISECOND, 0);
					end = helper.getTime();
					if (newDate.getTime() > endTime.getTime()) {
						renderer.createNewAfterClick(master, end, start, parent);
					} else {
						renderer.createNewAfterClick(master, start, end, parent);
					}
				} else {
					renderer.createNewAfterClick(master, start, parent);
				}
				startTimeString = null;
			}
			clearSelection();
		}
	}

	private void clearSelection() {
		for (int i = 0; i < hh.size(); i++) {
			if (hh.get(i) instanceof FocusPanel) {
				final FocusPanel walk = (FocusPanel) hh.get(i);
				walk.removeStyleName(FtrGwtDateCss.HOR_HOURS_SELECTED);
			}
		}
		startTimeString = null;
	}

	public M getMaster() {
		return master;
	}

	public void setMaster(M master) {
		this.master = master;
	}

	public int calculateIntervalSnapForX(int newHeight) {
		return calculator.calculateIntervalSnapForX(newHeight);
	}

}
