/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.horizontal;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import com.allen_sauer.gwt.dnd.client.DragContext;
import com.allen_sauer.gwt.dnd.client.drop.AbstractPositioningDropController;
import com.google.gwt.user.client.ui.Widget;

/**
 * DropController which allows a widget to be dropped on a SimplePanel drop target when the drop target does not yet
 * have a child widget.
 */
public class HorizontalViewPanelDropController<T, M> extends AbstractPositioningDropController {
	private Calendar helper = new GregorianCalendar(); // NOPMD;

	private HorizontalViewPanelNoDays<T, M> week = null;

	public HorizontalViewPanelDropController(HorizontalItemRow<T, M> newDropTarget, HorizontalViewPanelNoDays<T, M> newWeek) {
		super(newDropTarget);
		week = newWeek;
		// dropTargetInfo = new DropTargetInfo(newDropTarget.getBody());
	}

	@SuppressWarnings("unchecked")
	public void onDrop(DragContext context) {
		final Widget drp = getDropTarget();
		final Widget drag = context.draggable;
		if (drp instanceof HorizontalItemRow<?, ?> && drag instanceof HorizontalDayField<?, ?>) {
			final HorizontalItemRow<T, M> parent = (HorizontalItemRow<T, M>) drp;
			final HorizontalDayField<T, M> data = (HorizontalDayField<T, M>) drag;
			int x = context.draggable.getAbsoluteLeft() - week.getLeftScrollable();
			if (x < 0) {
				x = 0;
			}

			x = x + (parent.getRenderer().getStartHour() * parent.getRenderer().getIntervalsPerHour() * parent.getRenderer().getIntervalWidth());

			long duration = data.getEnd().getTime() - data.getStart().getTime();

			helper.setTime(data.getStart());
			// zero out time, keeping the date
			helper.set(Calendar.HOUR_OF_DAY, 0);
			helper.set(Calendar.MINUTE, 0);
			helper.set(Calendar.SECOND, 0);
			helper.set(Calendar.MILLISECOND, 0);
			// now add however many minutes the nearest interval represents
			helper.add(Calendar.MINUTE, (parent.calculateIntervalSnapForX(x)) * (60 / parent.getRenderer().getIntervalsPerHour()));
			data.setStart(helper.getTime());

			if (data.getEnd() != null) {
				data.setEndTime(new Date(data.getStart().getTime() + duration));
			}
			parent.addEventByDrop(data);
		}
	}

}
