package eu.future.earth.gwt.client.date.horizontal;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.allen_sauer.gwt.dnd.client.drop.AbstractDropController;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.DockLayoutPanel;
import com.google.gwt.user.client.ui.ProvidesResize;
import com.google.gwt.user.client.ui.ResizeComposite;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SimpleLayoutPanel;
import com.google.gwt.user.client.ui.Widget;

import eu.future.earth.gwt.client.FtrGwtDateCss;
import eu.future.earth.gwt.client.date.ComponentSizeEvent;
import eu.future.earth.gwt.client.date.ComponentSizehandler;
import eu.future.earth.gwt.client.date.DateEvent;
import eu.future.earth.gwt.client.date.DateEvent.DateEventActions;
import eu.future.earth.gwt.client.date.DateEventListener;
import eu.future.earth.gwt.client.date.EventPanel;
import eu.future.earth.gwt.client.date.HasComponentSizeEventHandlers;
import eu.future.earth.gwt.client.date.HasDateEventHandlers;

public class HorizontalViewPanelNoDays<T, M> extends ResizeComposite implements ProvidesResize, DateEventListener<T>, HasDateEventHandlers<T>, ComponentSizehandler {

	private ArrayList<HorizontalItemRow<T, M>> dayList = new ArrayList<HorizontalItemRow<T, M>>();

	private HorizontalDateRenderer<T, M> renderer = null;

	private ArrayList<AbstractDropController> dropControllers = new ArrayList<AbstractDropController>();

	private HorizontalViewPanelDragController itmeDragController = null;

	private AbsolutePanel itemsView = new AbsolutePanel();

	private ScrollPanel scroller = new ScrollPanel(itemsView);

	private AbsolutePanel labelView = new AbsolutePanel();

	private int labelWidth = 150;

	private boolean useCss = false;

	private int wide = 300;

	private DockLayoutPanel main = new DockLayoutPanel(Unit.PX);

	private SimpleLayoutPanel top = new SimpleLayoutPanel();

	private HorizontalTimeCalculator calculator;
	
	public HorizontalViewPanelNoDays(HorizontalDateRenderer<T, M> newRenderer, int newLabelWitd) {
		super();
		initWidget(top);
		top.add(main);
		labelWidth = newLabelWitd;
		renderer = newRenderer;
		calculator = new HorizontalTimeCalculator(renderer);
		if (renderer.enableDragAndDrop()) {
			itmeDragController = new HorizontalViewPanelDragController(itemsView);
		}

		if (useCss) {
			labelView.setStyleName(FtrGwtDateCss.HOR_LABELS);
		}
		main.addWest(labelView, labelWidth);
		main.add(scroller);
		// this.setWidget(0, 0, labelView);
		// this.getCellFormatter().setVerticalAlignment(0, 0, HorizontalPanel.ALIGN_TOP);
		// this.getCellFormatter().setHorizontalAlignment(0, 0, HorizontalPanel.ALIGN_RIGHT);
		// this.setWidget(0, 1, scroller);

		// this.getCellFormatter().setVerticalAlignment(0, 1, HorizontalPanel.ALIGN_TOP);
		// this.getCellFormatter().setHorizontalAlignment(0, 1, HorizontalPanel.ALIGN_LEFT);
		// this.setCellPadding(0);
		// this.setCellSpacing(0);
		redrawPanel();
	}

	private Widget topWidget;

	private Widget labelWidget;

	public void addWidgetToScroll(Widget newWidget, Widget newLabelWidget) {
		topWidget = newWidget;
		if (topWidget != null && topWidget instanceof HasComponentSizeEventHandlers) {
			
			HasComponentSizeEventHandlers real = (HasComponentSizeEventHandlers) topWidget;
			real.addComponentSizeHandler(this);
		}
		labelWidget = newLabelWidget;
		redrawPanel();
	}

	public int getTimePos(Date theDate) {
		return calculator.getTimePos(theDate);
	}
	
	public int getTimePos(int hourToCalc, int minutes) {
		return calculator.getTimePos(hourToCalc, minutes);
	}
	
	@SuppressWarnings("unchecked")
	public void clearData() {
		if (renderer.enableDragAndDrop()) {
			for (AbstractDropController dropController : dropControllers) {
				if (dropController instanceof HorizontalViewPanelDropController<?, ?>) {
					final HorizontalViewPanelDropController<T, M> real = (HorizontalViewPanelDropController<T, M>) dropController;
					itmeDragController.unregisterDropController(real);
				}
			}
			dropControllers.clear();
		}
		dayList.clear();
		redrawPanel();
	}

	// public void setPixelSize(int width, int height) {
	// setWidth(width);
	// }

	public void setWidth(int width) {
		if (width - labelWidth > 0) {
			scroller.setWidth(width - labelWidth + "px");
		}
	}

	public void notifyReady(DateEventActions action, T data, boolean reload) {
		if (reload) {
			loadData();
		}
		switch (action) {
		case ADD_DONE: {
			addEventData(data);
			break;
		}
		case REMOVE_DONE: {
			removeEventData(data);
			break;
		}
		case UPDATE_DONE: {
			updateEventData(data);
			break;
		}
		default:
			break;
		}
		DateEvent.fire(this, action, data);
	}

	public void loadData() {
		clearData();
		renderer.getEventsForRange(this);
	}

	public void loadEvents() {
		clearEvents();
		renderer.getEventsForRange(this);
	}

	public void clearEvents() {
		for (int i = 0; i < dayList.size(); i++) {
			final HorizontalItemRow<T, M> result = dayList.get(i);
			result.clearEvents();
		}
	}

	@SuppressWarnings("unchecked")
	public void removeEmptyRows() {
		for (int i = dayList.size() - 1; i > -1; i--) {
			final HorizontalItemRow<T, M> result = dayList.get(i);
			if (result.numberOfEvents() < 1) {
				if (renderer.enableDragAndDrop()) {
					Object dropController = dropControllers.get(i);
					if (dropController instanceof HorizontalViewPanelDropController<?, ?>) {
						final HorizontalViewPanelDropController<T, M> real = (HorizontalViewPanelDropController<T, M>) dropControllers.get(i);
						itmeDragController.unregisterDropController(real);
					}
				}
				dayList.remove(i);
				result.destroy();
			}
		}
		redrawPanel();
	}

	@SuppressWarnings("unchecked")
	public void removeRow(M theMaster, boolean redraw) {
		for (int i = dayList.size() - 1; i > -1; i--) {
			final HorizontalItemRow<T, M> result = dayList.get(i);
			if (renderer.isSameMaster(result.getMaster(), theMaster)) {
				if (renderer.enableDragAndDrop()) {
					Object dropController = dropControllers.get(i);
					if (dropController instanceof HorizontalViewPanelDropController<?, ?>) {
						final HorizontalViewPanelDropController<T, M> real = (HorizontalViewPanelDropController<T, M>) dropControllers.get(i);
						itmeDragController.unregisterDropController(real);
					}
				}
				dayList.remove(i);
				result.destroy();
			}
		}
		if (redraw) {
			redrawPanel();
		}
	}

	public List<? extends T> getRowData(M theMaster) {
		for (int i = dayList.size() - 1; i > -1; i--) {
			final HorizontalItemRow<T, M> result = dayList.get(i);
			if (renderer.isSameMaster(result.getMaster(), theMaster)) {
				return result.getData();
			}
		}
		return null;
	}

	public void addRows(List<M> newMaster) {
		for (M m : newMaster) {
			addRow(m);
		}
	}

	public HorizontalItemRow<T, M> addRow(M newMaster) {
		for (HorizontalItemRow<T, M> result : dayList) {
			if (result.isRowForMaster(newMaster)) {
				return null;
			}
		}
		HorizontalItemRow<T, M> newRow = new HorizontalItemRow<T, M>(renderer, this, newMaster);

		newRow.addDateEventHandler(this);
		if (renderer.enableDragAndDrop()) {
			final HorizontalViewPanelDropController<T, M> dayController = new HorizontalViewPanelDropController<T, M>(newRow, this);
			dropControllers.add(dayController);
			itmeDragController.registerDropController(dayController);
		}
		dayList.add(newRow);
		setHeight();
		int yPos = renderer.getRowHeight() * (dayList.size());
		if (topWidget != null) {
			yPos += getHeightCorrection();
		}
		newRow.setPixelSize(wide, renderer.getRowHeight());
		itemsView.add(newRow, 0, yPos);
		Widget currentLabel = renderer.getLabelForMaster(newRow.getMaster());
		currentLabel.setWidth(labelWidth - 2 + "px");
		if (useCss) {
			currentLabel.setStyleName(FtrGwtDateCss.HOR_ITEM_LABEL);
		}
		labelView.add(currentLabel, 0, yPos);
		return newRow;
	}

	public ArrayList<HorizontalItemRow<T, M>> getDayList() {
		return dayList;
	}

	public int getLeftScrollable() {
		return itemsView.getAbsoluteLeft();
	}

	// public void setPixelSize(int width, int height) {
	// setWidth(width);
	// }
	//
	// public void setWidth(int width) {
	// if (width - labelWidth > 0) {
	// scroller.setWidth(width - labelWidth + "px");
	// }
	// }

	private int getHeightCorrection() {
		if (topWidget != null) {
			GWT.log("Heigt top =" + topWidget.getOffsetHeight());
			return topWidget.getOffsetHeight();
		} else {
			return 0;
		}
	}

	public void setHeight() {
		int totalHeigt = (renderer.getRowHeight() * (dayList.size() + 1)) + getHeightCorrection();
		itemsView.setPixelSize(wide, totalHeigt);
		labelView.setPixelSize(labelWidth, totalHeigt);
		top.setHeight(totalHeigt + 18 + "px");
		// scroller.setHeight(totalHeigt + renderer.getRowHeight() + "px");
	}

	public void redrawPanel() {
		itemsView.clear();
		labelView.clear();
		wide = renderer.getIntervalsPerHour() * renderer.getIntervalWidth() * (renderer.getEndHour() - renderer.getStartHour());
		setHeight();
		int yPos = 0;

		final HorizontalHourPanel<T, M> hourPanel = new HorizontalHourPanel<T, M>(renderer, useCss);
		hourPanel.setPixelSize(wide, renderer.getRowHeight());
		itemsView.add(hourPanel, 0, yPos);
		yPos += renderer.getRowHeight();

		if (topWidget != null) {
			// topWidget.setPixelSize(wide, renderer.getRowHeight());
			itemsView.add(topWidget, 0, yPos);
			labelView.add(labelWidget, 0, yPos);
			yPos += getHeightCorrection();
		}

		for (HorizontalItemRow<T, M> current : dayList) {
			current.setPixelSize(wide, renderer.getRowHeight());
			itemsView.add(current, 0, yPos);
			Widget currentLabel = renderer.getLabelForMaster(current.getMaster());
			currentLabel.setWidth(labelWidth - 2 + "px");
			if (useCss) {
				currentLabel.setStyleName(FtrGwtDateCss.HOR_ITEM_LABEL);
			}
			labelView.add(currentLabel, 0, yPos);
			yPos += renderer.getRowHeight();
		}

		// super.setHeight((totalHeigt + renderer.getRowHeight()) + "px");
	}

	private void createPanel(HorizontalItemRow<T, M> result, T newEvent) {
		final EventPanel<T> entry = renderer.createPanel(newEvent);
		if (entry instanceof HorizontalDayField<?, ?>) {
			if (itmeDragController != null && entry.isDraggable()) {
				itmeDragController.makeDraggable(entry, entry.getDraggableItem());
			}
		}
		entry.addParentDateEventHandler(this);
		result.addEvent(entry);
	}

	public void handleDateEvent(DateEvent<T> newEvent) {
		switch (newEvent.getCommand()) {
		case DRAG_DROP: {
			renderer.updateEvent(renderer.createUpdateHandler(this, newEvent.getData()));
			break;
		}
		case ADD: {
			renderer.addEvent(renderer.createAddHandler(this, newEvent.getData()));
			break;
		}
		case UPDATE: {
			renderer.updateEvent(renderer.createUpdateHandler(this, newEvent.getData()));
			break;
		}
		case REMOVE: {
			renderer.removeEvent(renderer.createRemoveHandler(this, newEvent.getData()));
			break;
		}
		case RELOAD: {
			renderer.getEventsForRange(this);
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case ADD_DONE: {
			addEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case REMOVE_DONE: {
			removeEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case UPDATE_DONE: {
			updateEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case EDIT: {
			for (HorizontalItemRow<T, M> result : dayList) {
				if (result.isMaster(newEvent.getData())) {
					renderer.editAfterClick(result.getMaster(), newEvent.getData(), this);
				}
			}
			break;
		}
		default: {
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		}
	}

	public HandlerRegistration addDateEventHandler(DateEventListener<? extends T> handler) {
		return addHandler(handler, DateEvent.getType());
	}

	// private Calendar current = new GregorianCalendar();

	public void updateEventData(T newEvent) {
		removeEventData(newEvent);
		addEventData(newEvent);
	}

	public void removeEventData(T newEvent) {
		for (HorizontalItemRow<T, M> result : dayList) {
			EventPanel<T> removed = result.removeEvent(newEvent);
			if (removed != null) {
				removed.clearParent();
				return;
			}
		}

	}

	/**
	 * This method set's the events after clearing the data first.
	 * 
	 * @param newEvent
	 *            The Event list
	 */
	public void setEvents(List<T> newEvent) {
		clearData();
		if (newEvent != null) {
			for (T t : newEvent) {
				addEventData(t);
			}
		}
	}

	public void addEventData(T newEvent) {

		for (HorizontalItemRow<T, M> result : dayList) {
			if (result.isMaster(newEvent)) {
				createPanel(result, newEvent);
				return;
			}
		}
		// We did not found the master for this item. Ask for the master and add
		// it.
		renderer.getMaster(this, newEvent);

	}

	public void handleSizingEvent(ComponentSizeEvent newEvent) {
		redrawPanel();
		if (labelWidget != null && topWidget != null) {
			labelWidget.setPixelSize(labelWidth, topWidget.getOffsetHeight());
		}
	}

}
