/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.picker;

import java.util.Date;

import com.google.gwt.event.shared.GwtEvent;

public class DateSelectEvent extends GwtEvent<DateSelectListener> {

	private static final long serialVersionUID = -3798647759325157673L;

	public enum DateSelectEventActions {

		SELECT_DAY,

		SELECT_NEW_MONTH,
		
		NAVIGATE_MONTH,

	}

	private Date date = null;

	private DateSelectEventActions command = DateSelectEventActions.SELECT_DAY;

	public DateSelectEvent() {
		super();
	}

	/**
	 * Handler type.
	 */
	private static Type<DateSelectListener> TYPE;

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static DateSelectEvent fire(HasDateSelectHandlers source, Date navigator) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateSelectEvent event = new DateSelectEvent(navigator);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static DateSelectEvent fire(HasDateSelectHandlers source, DateSelectEvent navigator) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			source.fireEvent(navigator);
			return navigator;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static DateSelectEvent fire(HasDateSelectHandlers source, Date navigator, DateSelectEventActions action) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateSelectEvent event = new DateSelectEvent(navigator, action);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	/**
	 * Fires a before selection event on all registered handlers in the handler manager. If no such handlers exist, this
	 * method will do nothing.
	 * 
	 * @param source
	 *            the source of the handlers
	 * @param navigator
	 *            the navigator associated with this event
	 * @param token
	 *            the navigation token associated with this event
	 * @return the event so that the caller can check if it was canceled, or null if no handlers of this event type have
	 *         been registered
	 */
	public static DateSelectEvent fire(HasDateSelectHandlers source, DateSelectEventActions action) {
		// If no handlers exist, then type can be null.
		if (TYPE != null) {
			final DateSelectEvent event = new DateSelectEvent(action);
			source.fireEvent(event);
			return event;
		}
		return null;
	}

	public Date getDate() {
		return date;
	}

	public DateSelectEventActions getCommand() {
		return command;
	}

	/**
	 * Gets the type associated with this event.
	 * 
	 * @return returns the handler type
	 */
	public static Type<DateSelectListener> getType() {
		if (TYPE == null) {
			TYPE = new Type<DateSelectListener>();
		}
		return TYPE;
	}

	protected DateSelectEvent(Date newDate, DateSelectEventActions newAction) {
		super();
		date = newDate;
		command = newAction;
	}

	protected DateSelectEvent(DateSelectEventActions newAction) {
		super();
		command = newAction;
	}

	protected DateSelectEvent(Date newDate) {
		super();
		date = newDate;
	}

	@SuppressWarnings({
			"unchecked", "rawtypes"
	})
	@Override
	public final Type<DateSelectListener> getAssociatedType() {
		return (Type) TYPE;
	}

	protected void dispatch(DateSelectListener handler) {
		handler.handleDateSelectEvent(this);
	}

}
