/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.week;

import java.util.Calendar;

import com.google.gwt.user.client.ui.AbsolutePanel;
import com.google.gwt.user.client.ui.FocusPanel;
import com.google.gwt.user.client.ui.Widget;

import eu.future.earth.gwt.client.FtrGwtDateCss;
import eu.future.earth.gwt.client.date.DateRenderer;
import eu.future.earth.gwt.client.date.week.staend.ExtendedAbsolutePanel;

public abstract class BaseDayPanel<T> extends FocusPanel {

	protected DateRenderer<T> renderer;

	private ExtendedAbsolutePanel<T> body = null;

	public BaseDayPanel(DateRenderer<T> newRenderer) {
		super();
		this.renderer = newRenderer;
		body = new ExtendedAbsolutePanel<T>(this);
		this.setWidget(body);
		body.setStyleName(FtrGwtDateCss.DATE_DAY_FULL);
		super.setWidth(getPrefferedWitdh());
		super.setHeight(getNeededHeight() + "px");

	}

	public DateRenderer<T> getRenderer() {
		return renderer;
	}

	protected int getDailyIntervals() {
		return (renderer.getEndHour() - renderer.getStartHour()) * renderer.getIntervalsPerHour();
	}


	public int getNeededHeight() {
		return renderer.getIntervalHeight() * getDailyIntervals();
	}
	
	public boolean removeFromBody(Widget w) {
		return body.removeNormal(w);
	}

	public void add(Widget w, int left, int top) {
		body.addNormal(w, left, top);
	}


	public AbsolutePanel getBody() {
		return body;
	}

	public abstract void repaintEvents();

	protected abstract String getPrefferedWitdh();

	public abstract void setDay(Calendar helperCal);

}
