package eu.future.earth.gwt.client.date.week;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.RequiresResize;
import com.google.gwt.user.client.ui.ResizeComposite;

import eu.future.earth.gwt.client.FtrGwtDateCss;
import eu.future.earth.gwt.client.date.DateEvent;
import eu.future.earth.gwt.client.date.DateEventListener;
import eu.future.earth.gwt.client.date.DatePanel;
import eu.future.earth.gwt.client.date.DateRenderer;
import eu.future.earth.gwt.client.date.DateUtils;
import eu.future.earth.gwt.client.date.EventPanel;
import eu.future.earth.gwt.client.date.HasDateEventHandlers;
import eu.future.earth.gwt.client.date.DateEvent.DateEventActions;

public abstract class BaseWeekPanel<T> extends ResizeComposite implements HasDateEventHandlers<T>, DatePanel<T>, DateEventListener<T>, RequiresResize {

	private ArrayList<DayEventElement<T>> dayList = new ArrayList<DayEventElement<T>>();

	

	private HashMap<Integer, DayHeader<T>> cacheDayHeaders = new HashMap<Integer, DayHeader<T>>();

	private HashMap<Integer, DayEventpanel<T>> cacheDaysEventPanels = new HashMap<Integer, DayEventpanel<T>>();

	public BaseWeekPanel(int newNumberOfDays, DateRenderer<T> newRenderer, boolean newShowDayLabel) {
		super();
		showDayLabel = newShowDayLabel;
		daysShown = newNumberOfDays;
		renderer = newRenderer;
		current.setFirstDayOfWeek(firstDayOfWeek);
		current.setTime(new Date());
		current.set(Calendar.HOUR_OF_DAY, 0);
		current.set(Calendar.MINUTE, 0);
		current.set(Calendar.SECOND, 0);
		current.set(Calendar.MILLISECOND, 0);
	}

	protected int daysShown = 7;

	protected int firstDayOfWeek = Calendar.MONDAY;

	protected Date firstLogical = null;

	protected Date lastLogical = null;

	private boolean showDayLabel = true;

	public boolean isShowDayLabel() {
		return showDayLabel;
	}

	protected int getDailyIntervals() {
		return (renderer.getEndHour() - renderer.getStartHour()) * renderer.getIntervalsPerHour();
	}

	public int getNeededHeight() {
		return renderer.getIntervalHeight() * getDailyIntervals();
	}

	protected Calendar current = new GregorianCalendar();

	public int getFirstDayOfWeek() {
		return firstDayOfWeek;
	}

	protected DateRenderer<T> renderer = null;

	public DateRenderer<T> getRenderer() {
		return renderer;
	}

	public void setDate(Date activeday) {
		GWT.log("Setting to " + activeday, null);
		final Calendar test = new GregorianCalendar();
		test.setTime(activeday);
		test.set(Calendar.HOUR_OF_DAY, 0);
		test.set(Calendar.MINUTE, 0);
		test.set(Calendar.SECOND, 0);
		test.set(Calendar.MILLISECOND, 0);
		if (DateUtils.isSameDay(test, current)) {
			return;
		}
		current.setTime(activeday);
		current.set(Calendar.HOUR_OF_DAY, 0);
		current.set(Calendar.MINUTE, 0);
		current.set(Calendar.SECOND, 0);
		current.set(Calendar.MILLISECOND, 0);
		// buildPanel();
		modifyDate();
	}

	public void setRenderer(DateRenderer<T> renderer) {
		this.renderer = renderer;
		buildPanel();
	}

	public void setDaysShown(int newDaysShown) {
		if (daysShown == newDaysShown) {
			return;
		}
		this.daysShown = newDaysShown;
		buildPanel();
	}

	public abstract void buildPanel();

	public void setFirstDayOfWeek(int newFirstDayOfWeek) {
		if (firstDayOfWeek == newFirstDayOfWeek) {
			return;
		}
		this.firstDayOfWeek = newFirstDayOfWeek;
		current.setFirstDayOfWeek(firstDayOfWeek);
		buildPanel();
	}

	private void modifyDate() {
		clearEvents();
		Calendar helperCal = new GregorianCalendar();
		helperCal.setFirstDayOfWeek(getFirstDayOfWeek());
		helperCal.setTime(current.getTime());
		if (daysShown > 1) {
			helperCal.set(Calendar.DAY_OF_WEEK, helperCal.getFirstDayOfWeek());
		}
		GWT.log("Start on " + helperCal.getTime(), null);
		firstLogical = helperCal.getTime();
		for (int i = 0; i < daysShown; i++) {
			int dayOfWeek = helperCal.get(Calendar.DAY_OF_WEEK);
			setDateForPanels(dayOfWeek, helperCal);
			helperCal.add(Calendar.DAY_OF_WEEK, 1);
		}
		lastLogical = helperCal.getTime();

	}

	public int getDaysShown() {
		return daysShown;
	}

	public void clearCache() {
		cacheDayPanels.clear();
		cacheDayHeaders.clear();
		cacheDaysEventPanels.clear();
		for (int i = 0; i < dayList.size(); i++) {
			final DayEventElement<T> result = dayList.get(i);
			result.destroy();
			result.clearEvents();
		}
		dayList.clear();
	}

	public void put(int dayOfWeek, BaseDayPanel<T> dayPanel) {
		cacheDayPanels.put(dayOfWeek, dayPanel);
	}

	public void put(int dayOfWeek, DayHeader<T> label) {
		cacheDayHeaders.put(dayOfWeek, label);
	}

	private HashMap<Integer, BaseDayPanel<T>> cacheDayPanels = new HashMap<Integer, BaseDayPanel<T>>();
	
	public void put(int dayOfWeek, DayEventpanel<T> eventPanel) {
		cacheDaysEventPanels.put(dayOfWeek, eventPanel);
		dayList.add(eventPanel);
	}

	public void add(DayEventElement<T> dayPanel) {
		dayList.add(dayPanel);
	}

	public void setDateForPanels(int dayOfWeek, Calendar helperCal) {
		final Calendar today = new GregorianCalendar();
		if (cacheDayPanels.size() == 1) {
			dayOfWeek = cacheDayPanels.keySet().iterator().next();
		}
		{
			BaseDayPanel<T> dayPanel = (BaseDayPanel<T>) cacheDayPanels.get(dayOfWeek);
			if (dayPanel != null) {
				dayPanel.setDay(helperCal);
				if (DateUtils.isSameDay(today, helperCal)) {
					dayPanel.addStyleName(FtrGwtDateCss.DATE_DAY_TODAY);
				} else {
					dayPanel.removeStyleName(FtrGwtDateCss.DATE_DAY_TODAY);
				}
			}
		}
		{
			final DayHeader<T> label = (DayHeader<T>) cacheDayHeaders.get(dayOfWeek);
			if (label != null) {
				label.setDate(helperCal.getTime());
				if (DateUtils.isSameDay(today, helperCal)) {
					label.addStyleName(FtrGwtDateCss.DATE_DAY_TODAY);
				} else {
					label.removeStyleName(FtrGwtDateCss.DATE_DAY_TODAY);
				}
			}
		}
		final DayEventpanel<T> eventPanel = (DayEventpanel<T>) cacheDaysEventPanels.get(dayOfWeek);
		if (eventPanel != null) {
			eventPanel.setDay(helperCal);
		}
	}

	public void clearEvents() {
		if (dayList != null) {
			for (int i = 0; i < dayList.size(); i++) {
				final DayEventElement<T> result = dayList.get(i);
				result.clearEvents();
			}
		}
	}

	public Date next() {
		if (daysShown > 1) {
			current.add(Calendar.DAY_OF_WEEK, 7);
		} else {
			current.add(Calendar.DAY_OF_WEEK, 1);
		}
		// current.add(Calendar.DAY_OF_WEEK, daysShown);
		// buildPanel();
		modifyDate();
		return current.getTime();
	}

	public Date prev() {
		if (daysShown > 1) {
			current.add(Calendar.DAY_OF_WEEK, -7);
		} else {
			current.add(Calendar.DAY_OF_WEEK, -1);
		}
		// buildPanel();
		modifyDate();
		return current.getTime();
	}

	public Date nextMore() {
		if (daysShown > 1) {
			current.add(Calendar.MONTH, 1);
		} else {
			current.add(Calendar.DAY_OF_WEEK, 7);
		}
		// buildPanel();
		modifyDate();
		return current.getTime();
	}

	public Date prevMore() {
		if (daysShown > 1) {
			current.add(Calendar.MONTH, -1);
		} else {
			current.add(Calendar.DAY_OF_WEEK, -7);
		}
		// buildPanel();
		modifyDate();
		return current.getTime();
	}

	public void updateEventData(T newEvent) {
		removeEventData(newEvent);
		addEventData(newEvent);
	}

	public void removeEventData(T newEvent) {
		for (DayEventElement<T> panel : dayList) {
			EventPanel<? extends T> removed = panel.removeEvent(newEvent);
			if (removed != null) {
				removed.clearParent();
				return;
			}
		}

	}

	public void addEventsByList(List<? extends T> events) {
		if (events != null) {
			for (int i = 0; i < events.size(); i++) {
				addEventData(events.get(i), true);
			}
		}

		for (DayEventElement<T> panel : dayList) {
			panel.repaintEvents();
		}
	}

	private void addEventData(T newEvent, boolean isPartOfBatch) {
		for (DayEventElement<T> panel : dayList) {
			if (panel.isDay(newEvent)) {
				createPanel(panel, newEvent, isPartOfBatch);
				panel.repaintEvents();
			}
		}
	}

	public abstract void createPanel(DayEventElement<T> panel, T newEvent, boolean isPartOfBatch);

	public void addEventData(T newEvent) {
		addEventData(newEvent, false);
	}

	@Override
	public void onResize() {
		super.onResize();
		repaint();
	}

	public void repaint() {
		for (DayEventElement<T> panel : dayList) {
			panel.repaintEvents();
		}
	}

	public Date getFirstDateShow() {
		return firstLogical;
	}

	public Date getLastDateShow() {
		return lastLogical;
	}

	public Date getFirstDateLogical() {
		return firstLogical;
	}

	public Date getLastDateLogical() {
		return lastLogical;
	}

	public void clearData() {
		if (dayList != null) {
			for (DayEventElement<T> panel : dayList) {
				panel.clearEvents();
			}
		}
	}

	public void setEvents(List<? extends T> events) {
		clearData();
		addEventsByList(events);
	}

	public void notifyReady(DateEventActions action, T data, boolean reload) {
		if (reload) {
			loadData();
		}
		switch (action) {
		case ADD_DONE: {
			addEventData(data);
			break;
		}
		case REMOVE_DONE: {
			removeEventData(data);
			break;
		}
		case UPDATE_DONE: {
			updateEventData(data);
			break;
		}
		default:
			break;
		}
		DateEvent.fire(this, action, data);
	}

	public void loadData() {
		clearData();
		renderer.getEventsForRange(getFirstDateShow(), getLastDateShow(), this, true);
	}

	public void handleDateEvent(DateEvent<T> newEvent) {
		switch (newEvent.getCommand()) {
		case DRAG_DROP: {
			renderer.updateEvent(renderer.createUpdateHandler(this, newEvent.getData()));
			break;
		}
		case UPDATE: {
			renderer.updateEvent(renderer.createUpdateHandler(this, newEvent.getData()));
			break;
		}
		case REMOVE: {
			renderer.removeEvent(renderer.createRemoveHandler(this, newEvent.getData()));
			break;
		}
		case ADD: {
			renderer.addEvent(renderer.createAddHandler(this, newEvent.getData()));
			break;
		}
		case ADD_DONE: {
			addEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case REMOVE_DONE: {
			removeEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case UPDATE_DONE: {
			updateEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case EDIT: {
			renderer.editAfterClick(newEvent.getData(), this);
			break;
		}
		case RELOAD: {
			loadData();
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		default:
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
	}

	public HandlerRegistration addDateEventHandler(DateEventListener<? extends T> handler) {
		return addHandler(handler, DateEvent.getType());
	}

	public HandlerRegistration addWeekScrollEventHandler(WeekScrollEventHandler handler) {
		return super.addHandler(handler, WeekScrollEvent.getType());
	}

}
